REBOL [
	Title: "REBOL Mezzanine Functions: Config"
	Rights: "Copyright REBOL Technologies 1997-2002"
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

write-user: func [
	"Write network config to user.r file"
	/local user.r
][
	; to be changed in the future with something like system/options/data/user.r
	user.r: either view? [view-root/user.r] [system/options/home/user.r]
	write user.r trim reform [{
		REBOL [
			Title: "User Preferences"
			Date: } now {
		]

		set-net [}
			system/user/email
			system/schemes/default/host
			system/schemes/pop/host
			system/schemes/default/proxy/host
			system/schemes/default/proxy/port-id
			system/schemes/default/proxy/type
			; TODO: ESMTP username/password
		{]
		} either found? system/user/name [
			reform ["set-user-name" mold system/user/name newline]
			][#]
		either view? [
			reform ["if (not none? system/view) [^/^-system/view/screen-face/options:"
				mold system/view/screen-face/options "^/]^/"]
		] [#]
	]
]

save-user: func [
	"Save user.r, prompting for overwrite permission"
	/local user.r
][
	user.r: either view? [view-root/user.r] [system/options/home/user.r]
	if any [
		not exists? user.r 
		confirm {^/Overwrite the current user.r file? }
	][
		print {^/Creating user.r file in the REBOL home directory.^/}
		write-user
		print trim {
			To change these settings, type "set-user" at the
			prompt or edit your user.r file.
		}
	]
]

set-user-name: func [str [string!]] [system/user/name: str]

link-app?: func [
	"Tell whether a script is running under a Link application context."
] [
	none
]

link?: func [
	"Returns true if REBOL/Link capability is enabled."
][
	found? component? 'link
]

protect-system: func [
	{Protects all system functions and the system object from redefinition.}
	/local vals words word 
][
	vals: second system/words
	words: make block! 1
	foreach word first system/words [
		if any-function? first vals [ append words word ]
		vals: next vals
	]
	foreach word bind words 'read [ protect word ]
	protect 'system
]

