REBOL [
	Title: "REBOL/View Desktop - Program Launcher"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

do-file: func [path stats /local orig] [
	; Handle an icon action. May be a static word action, email address,
	; URL, or file.  If file, load it and perform the appropriate action.
	if none? path [exit]
	orig: path
	if word? path [
		switch path [
			goto [goto-view]
			help [help-view]
			console [halt-view]
			quit [quit]
		]
		exit
	]
	if email? path [emailer/to path exit]
	; If path is a URL or absolute file path, do not prepend folder path.
	if not any [url? path (first path) = #"/"] [path: full-path path]
	either not read-binary path stats [
		alert reform ["Missing file:" path]
	][
		switch/default map-suffix to-file path [
			rebol [start path]
			text [editor path-thru path]
			image [show-image path]
		][
			either url? orig [browse orig][browse path-thru path]
		]
	]
]	

start: function [path][p file][
	; Startup another copy of REBOL to run an application:
	tell reform ["Launching" path]
	if url? path [
		either any [error? try [launch/secure-cmd path-thru path]][
			alert reform ["Cannot launch application:" path]
		]
		exit
	]
	if file? path [
		if not exists? path [alert reform ["File does not exist:" path] exit]
		if error? try [launch/secure-cmd path] [
			alert reform ["Cannot launch application:" path]
		]
	]
]

show-image: func [path /local face] [
	; Show an image in a window on the screen:
	path: load-binary path true
	if image? path [
		view/new center-face make-face/spec 'image [
			image: path
			size: min image/size system/view/screen-face/size
		]
	]
]

read-binary: func [url info] [
	; Read a binary file from the net through the cache: 
	; info: none=do a reload, true=use local copy, block of stats=check
	dbug ["Read-binary" url info]
	; if not online or file is cached, get it.
	return either any [not alive? all [info exists-thru?/check url info]] [
		; Allow reading a stale file if off-line:
		if all [url? url  not alive?  not exists-thru? url] [return none]
		; Indicate we are doing something!
		show-progress 1 0
		info: read-thru/progress/expand url :show-progress
		show-progress 1 1
		info
	][
		; Indicate we are doing something!
		show-progress 1 0
		info: read-thru/progress/expand/check/update url :show-progress info
		show-progress 1 1
		info
	]
]

load-binary: func [url fresh] [
	; Load a file from the net thru the cache:
	if url: read-binary url fresh [
		if error? try [url: load url][return none]
		return url
	]
]

do-local: has [files] [
	; Execute a local file (CTRL-L shortcut):
	files: request-file/keep 
	if none? files [exit] 
	foreach file files [do-file file none]
] 

halt-view: does [
	; Close the desktop and open the console:
	unview/all
	about
	halt
]

map-suffix: func [file] [
	; Return the datatype word based on a file's suffix:
	all [
		file? file
		n: find/last file "."
		n: find suffix-map n  ;copy n ?
		n: find n word!
		pick n 1
	]
]

