REBOL [
	Title: "REBOL/View Desktop - Preferences GUI"
	Version: 1.0.0
	Author: "Gabriele Santilli"
	Rights: "Copyright REBOL Technologies 2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

ctx-prefs-gui: [

; Set GUI fields from the user prefs
prefs-to-face: does [
	if function? get in system/schemes/default/proxy 'user [
		system/schemes/default/proxy/user:
		system/schemes/default/proxy/pass: none
	]
	foreach [field value] [
		field-name      [safe-copy user-prefs/name]
		check-connect   user-prefs/auto-connect
		check-desk      user-prefs/desktop
		field-email     [either email? system/user/email [form system/user/email] [copy ""]]
		field-smtp      [safe-copy system/schemes/default/host]
		field-pop       [safe-copy system/schemes/pop/host]
		choice-ptype    [
			select reduce [none "None" 'none "None" 'generic "Generic" 'socks4 "Socks 4" 'socks "Socks 5"]
				system/schemes/default/proxy/type
		]
		field-phost     [safe-copy system/schemes/default/proxy/host]
		field-pport     [either integer? system/schemes/default/proxy/port-id [form system/schemes/default/proxy/port-id] [copy ""]]
		field-puser     [safe-copy system/schemes/default/proxy/user]
		field-ppass     [safe-copy system/schemes/default/proxy/pass]
	] [
		set-face get field do value
	]
]
safe-copy: func [value] [copy either string? value [value] [""]]
safe-to-block: func [value] [any [attempt [to block! value] [ ]]]
type-check: func [field rule /local value] [
	parse safe-to-block get-face field bind rule 'value
	:value
]
; Set the user prefs from the GUI fields
face-to-prefs: has [user pass] [
	system/user/name: user-prefs/name: safe-copy get-face field-name
	user-prefs/auto-connect: to logic! get-face check-connect
	user-prefs/desktop: to logic! get-face check-desk
	system/user/email: type-check field-email [set value email!]
	system/schemes/default/host: safe-copy get-face field-smtp
	system/schemes/pop/host: safe-copy get-face field-pop
	system/schemes/default/proxy/type: select reduce [
		"None" none "Generic" 'generic "Socks 4" 'socks4 "Socks 5" 'socks
	] get-face choice-ptype
	either system/schemes/default/proxy/type [
		system/schemes/default/proxy/host: safe-copy get-face field-phost
		system/schemes/default/proxy/port-id: type-check field-pport [set value integer!]
		user: safe-copy get-face field-puser
		pass: safe-copy get-face field-ppass
		if all [not empty? user not empty? pass] [
			system/schemes/default/proxy/user: user
			system/schemes/default/proxy/pass: pass
		]
	] [
		system/schemes/default/proxy/host:
		system/schemes/default/proxy/port-id:
		system/schemes/default/proxy/user:
		system/schemes/default/proxy/pass: none
	]
]

field-name: check-connect: check-desk: field-email: field-smtp: field-pop: 
choice-ptype: field-phost: field-pport: field-puser: field-ppass: none
install-now: none
slide-face: tmp: none

width: 540

user-settings: 0
email-settings: negate width
proxy-settings: negate width * 2

; Sliding pane effect
slide-to: func [offset] [
	if offset = slide-face/offset/x [exit]
	slide-face/user-data: context [
		dest: offset
		midpoint: offset + slide-face/offset/x / 2
		speed: sign? offset - slide-face/offset/x
		accel: (abs offset - slide-face/offset/x) / 50 * speed
		op: either dest < slide-face/offset/x [:lesser-or-equal?] [:greater-or-equal?]
	]
	slide-face/rate: 25 show slide-face
]

slide-engage: func [face action event] [
	if action = 'time [
		face/offset/x: face/offset/x + face/user-data/speed
		if face/user-data/op face/offset/x face/user-data/dest [
			face/rate: none
			face/offset/x: face/user-data/dest
			show face
			exit
		]
		if face/user-data/op face/offset/x face/user-data/midpoint [
			face/user-data/accel: negate face/user-data/accel
		]
		face/user-data/speed: face/user-data/speed + face/user-data/accel
		show face
	]
]

; GUI layout
prefs-face: layout [
	style tog tog of 'slide 100 snow 255.255.200
	backeffect base-effect
	origin 10
	;size 540x385
	across 
	h2 "REBOL Preferences"
	pad 30
	tmp: tog "User settings"  [slide-to user-settings] do [tmp/data: tmp/state: on]
	tog "Email settings" [slide-to email-settings]
	tog "Proxy settings" [slide-to proxy-settings]
	below
	slide-face: panel 540x220 [
		size 1620x220
		style rtxt text 100x24 bold middle right
		style cmt text 230x24 italic middle
		;style clabel text bold 200x16 para none
		;style link text italic underline font [colors: reduce [black over-color]]
		style field field 190 edge [color: dtw-skin/main-color]
		style drop-down drop-down 190 edge [color: dtw-skin/main-color]
		style h2 h2 200x32
		space 4

		backdrop dtw-skin/main-color effect dtw-skin/main-effect
		across

		at 20x15 guide
		h2 "User settings:" return
		rtxt "User name:" field-name: field cmt "Enter your name here" return
		pad 6x12
		rtxt    check-desk: check-line on bold "Open desktop on startup" return
		pad 6x-4
		rtxt check-connect: check-line on bold "Auto-connect on startup" return
		pad 6x10
		rtxt install-now: check-line off bold "Install now (options will be shown)" return
	
		;rtxt link "Email settings" [slide-to email-settings] return
		;rtxt link "Proxy settings" [slide-to proxy-settings] return

		at 560x15 guide
		h2 "Email settings:" return
		rtxt "Email address:" field-email: field cmt "In order to send email" return
		rtxt   "SMTP server:"  field-smtp: field cmt "Your email server: mail.example.com" return
		rtxt    "POP server:"   field-pop: field cmt "Often the same as above" return
		rtxt cmt "All fields are optional."
		;rtxt link "User settings" [slide-to user-settings] return
		;rtxt link "Proxy settings" [slide-to proxy-settings] return

		at 1100x15 guide
		h2 "Proxy settings:" return 
		rtxt   "Proxy type:" choice-ptype: drop-down "None" "Generic" "Socks 4" "Socks 5" return
		pad 0x10
		rtxt "Proxy server:"  field-phost: field      cmt "Example: proxy.example.com" return
		rtxt   "Proxy port:"  field-pport: field      cmt "Port number. Example: 1080" return
		pad 0x10
		rtxt     "Username:"  field-puser: field      cmt "Optional proxy username" return
		rtxt     "Password:"  field-ppass: field hide cmt "Optional proxy password" return
		;rtxt link "User settings" [slide-to user-settings] return
		;rtxt link "Email settings" [slide-to email-settings] return
	] edge [size: 0x1 color: base-color effect: 'ibevel] feel [engage: :slide-engage]
	do [slide-face/offset/x: 0]
	do [slide-face/size/x: width * 3]
	across
	pad 10 
	text 100 navy underline "Help" [browse http://www.rebol.com/docs/desktop.html]
	pad 220
	btn-enter 72 "Save" [do-save]
	btn-cancel 72 "Cancel" [hide-popup]
]
prefs-face/size/x: width

do-save: does [
	face-to-prefs
	if error? try [ctx-prefs/save-prefs] [alert "Unable to save settings in your prefs.r file."]
	ctx-prefs/save-user
	;unview/only prefs-face
	hide-popup
	if get-face install-now [install]
]

view-prefs: does [
	prefs-to-face
	inform prefs-face
]

]

view-prefs: does [
	if block? ctx-prefs-gui [ctx-prefs-gui: context ctx-prefs-gui]
	ctx-prefs-gui/view-prefs
]

set 'set-user :view-prefs
;view-prefs